@@ -11,7 +11,7 @@ use strict;
 @ISA = qw(Verilog::Netlist::Net::Struct
 	Verilog::Netlist::Subclass);
 
-$VERSION = '3.422';
+$VERSION = '3.448';
 
 my %_Type_Widths = (
     'bit'	=> 1,
@@ -190,7 +190,10 @@ sub lint {
     }
     elsif ($self->_used_out()) {
 	if ($self->_used_out()>1
-	    && !$self->array()) {   # if an array, different outputs might hit different bits
+	    # if an array, different outputs might hit different bits
+	    && !$self->array()
+	    # if vector, warn only if # of usages is higher than # of bits in vector
+	    && (abs($self->msb() - $self->lsb()) + 1) < $self->_used_out()) {
 	    $self->warn("Signal has multiple drivers (",
 			$self->_used_out(),"): ",$self->name(), "\n");
 	    $self->dump_drivers(8);
@@ -261,14 +264,17 @@ sub dump_drivers {
     }
     foreach my $cell ($self->module->cells_sorted) {
 	foreach my $pin ($cell->pins_sorted) {
-	    if ($pin->port && $pin->net && $pin->net == $self) {
-		print " "x$indent,"  Pin:  ",$cell->name,".",$pin->name
-		    ,"  ",$pin->port->direction,"\n";
-	    }
-	    elsif ($pin->net && $self->name eq $pin->net->name) {
-		warn "%Warning: Internal net name duplicate: ".$cell->name."  ".$self->name."\n"
-		    .$self->comment."  ".$pin->net->comment."\n"
-		    ."$self  ".$pin->net->name."\n";
+	    foreach my $net ($pin->nets) {
+		next unless defined $net->{net};
+		if ($pin->port && $net->{net} == $self) {
+		    print " "x$indent,"  Pin:  ",$cell->name,".",$pin->name
+			,"  ",$pin->port->direction,"\n";
+		}
+		elsif ($self->name eq $net->{net}->name) {
+		    warn "%Warning: Internal net name duplicate: ".$cell->name."  ".$self->name."\n"
+			.$self->comment."  ".$net->{net}->comment."\n"
+			."$self  ".$net->{net}->name."\n";
+		}
 	    }
 	}
     }
@@ -309,8 +315,9 @@ See also Verilog::Netlist::Subclass for additional accessors and methods.
 
 =item $self->array
 
-Any array (vector) declaration for the net.  This is for multidimensional
-signals, for the width of a signal, use msb/lsb/width.
+Any array (vector) declaration for the net.  This is for Verilog 2001
+multidimensional signals; for the width of a signal, use msb/lsb/width. For
+newer applications use data_type() as it supports SystemVerilog types.
 
 =item $self->comment
 
@@ -401,7 +408,7 @@ Verilog-Perl is part of the L<http://www.veripool.org/> free Verilog EDA
 software tool suite.  The latest version is available from CPAN and from
 L<http://www.veripool.org/verilog-perl>.
 
-Copyright 2000-2016 by Wilson Snyder.  This package is free software; you
+Copyright 2000-2018 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 
