@@ -11,7 +11,7 @@ use strict;
 @ISA = qw(Verilog::Netlist::Port::Struct
 	Verilog::Netlist::Subclass);
 
-$VERSION = '3.430';
+$VERSION = '3.448';
 
 structs('_new_base',
 	'Verilog::Netlist::Port::Struct'
@@ -67,12 +67,19 @@ sub _link {
     if (!$self->net) {
 	my $net = $self->module->find_net ($self->name);
 	if (!$net) {
+	    my $msb;
+	    my $lsb;
+	    if (defined $self->data_type) {
+		$self->data_type =~ /\[([^:]+)(:(.*))?\]$/;
+		$msb = $1;
+		$lsb = defined($3) ? $3 : $1;
+	    }
 	    $net = $self->module->new_net
 		(name=>$self->name,
 		 filename=>$self->filename, lineno=>$self->lineno,
 		 decl_type=>"port", net_type=>"wire",
 		 data_type=>$self->data_type, array=>$self->array,
-		 comment=>undef,
+		 comment=>undef, msb=>$msb, lsb=>$lsb,
 		 );
 	    $net->attributes($self->attributes);  # Copy attributes across
 	}
@@ -136,8 +143,8 @@ See also Verilog::Netlist::Subclass for additional accessors and methods.
 Any array declaration for the port.  This only applies to Verilog 1995
 style ports which can declare port bits independently from the signal
 declarations.  When using Verilog 2001 style ports, see the matching net
-declaration's lsb and msb methods instead, for example
-C<$module->find_net($port->name)->msb>.
+declaration's data_type, msb and lsb methods instead, for example
+C<$module->find_net($port->name)->data_type>.
 
 =item $self->comment
 
@@ -190,7 +197,7 @@ Verilog-Perl is part of the L<http://www.veripool.org/> free Verilog EDA
 software tool suite.  The latest version is available from CPAN and from
 L<http://www.veripool.org/verilog-perl>.
 
-Copyright 2000-2017 by Wilson Snyder.  This package is free software; you
+Copyright 2000-2018 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 
