@@ -14,7 +14,7 @@ use base qw(DynaLoader);
 use strict;
 use vars qw($VERSION $Debug);
 
-$VERSION = '3.422';
+$VERSION = '3.448';
 
 #$Debug sets the default value for debug.  You're better off with the object method though.
 
@@ -58,6 +58,7 @@ sub new {
 		use_vars => 1,
 		use_unreadback => 1,   # Backward compatibility
 		use_protected => 1,   # Backward compatibility
+		use_pinselects => 0,   # Backward compatibility
 		use_std => undef,	# Undef = silent
 		#use_cb_{callback-name} => 0/1
 		#
@@ -72,6 +73,7 @@ sub new {
 		$self->{_sigparser},
 		$self->{use_unreadback},
 		$self->{use_protected},
+		$self->{use_pinselects},  # Undocumented as for use in SigParser only
 		);
 
     $self->{use_cb_contassign} = $self->{use_vars} if !exists $self->{use_cb_contassign};
@@ -329,8 +331,8 @@ C and Perl to invoke the callbacks.  When using this feature,
 use_unreadback=>0 should be used too, as since whole tokens are skipped,
 skipping whitespace shouldn't matter either.
 
-Adding "use_protected => 0" will disable callbacks on `protected regions,
-which may improve performance.
+Adding "use_protected => 0" will disable callbacks on `protected and
+"`pragma protect protected" regions, which may improve performance.
 
 Adding "use_std => 1" will add parsing of the SystemVerilog built-in std::
 package, or "use_std => 0" will disable it.  If unspecified it is silently
@@ -525,7 +527,7 @@ Verilog-Perl is part of the L<http://www.veripool.org/> free Verilog EDA
 software tool suite.  The latest version is available from CPAN and from
 L<http://www.veripool.org/verilog-perl>.
 
-Copyright 2000-2016 by Wilson Snyder.  This package is free software; you
+Copyright 2000-2018 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 
