@@ -11,7 +11,7 @@ use base qw(DynaLoader);
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '3.418';
+$VERSION = '3.448';
 
 ######################################################################
 #### Configuration Section
@@ -84,6 +84,13 @@ sub DESTROY {
     $self->_DESTROY;
 }
 
+sub STORABLE_freeze {
+    my ($self, $cloning) = @_;
+    # Prevent crash on Storable::store then retrieve
+    delete $self->{_cthis};
+    return;
+}
+
 sub open {
     my $self = shift;
     my %params = (
@@ -458,7 +465,7 @@ Verilog-Perl is part of the L<http://www.veripool.org/> free Verilog EDA
 software tool suite.  The latest version is available from CPAN and from
 L<http://www.veripool.org/verilog-perl>.
 
-Copyright 2000-2016 by Wilson Snyder.  This package is free software; you
+Copyright 2000-2018 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 
